;; Generated package description from yaml-mode.el  -*- no-byte-compile: t; lexical-binding:t -*-
(define-package "yaml-mode" "0.0.16.0.20260809.15" "Major mode for editing YAML files" '((emacs "24.1")) :commit "5d7347519c94eac1c9383a3b9bebe218d3e43318" :authors '(("Yoshiki Kurihara" . "clouder@gmail.com") ("Marshall T. Vandegrift" . "llasram@gmail.com")) :maintainer '("Vasilij Schneidermann" . "mail@vasilij.de") :keywords '("data" "yaml") :url "https://github.com/yoshiki/yaml-mode")
